﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;

using Newtonsoft.Json.Linq;

/*
 * 
    Upload a file you want to use as input using the WorkFile API.
    Start a conversion operation by using the POST URL below.
    Check the status of the conversion by (repeatedly) using the GET URL below.
    When complete, a separate output file will exist which you can download via the WorkFile API.

    Set up command line arguments for adding the files
 * 
 * */


namespace PrizmDocConvertHTMLtoPDF
{
    class Program
    {

        static string ApiKey;

        static void LoadConfig()
        {
            using (var reader = new StreamReader("config.json"))
            {
                dynamic config = JObject.Parse(reader.ReadToEnd());
                ApiKey = config.apiKey;
            }
        }


        /*
         * ConvertFileToPDF uses the PrizmDocConvert class to:
         * 1.  Upload the file to the PrizmDoc cloud with the user's API key
         * 2.  Issue a convert command on the file to convert it to PDF
         * 3.  Download the file and save is as output file name.  
         * 
         * Please note this method contains to true error correction - if you convert a file to pdf
         * and save it as a .gif, that's on you.
         * 
         * References:
         * string input:  The file to be uploaded and converted
         * string output: The file name to save the converted file as
         * bool verbuse: Default: False.  This determines if console messages are issues.
         * 
         * 
         * */

        static bool ConvertFileToPDF(string input, string output, bool verbose=false)
        {
            if (verbose)
                Console.WriteLine("Getting the API Key from config.json.");

            //if the input file doesn't exist then stop there
            if (!(File.Exists(input)))
            {
                return false;
            }
            //load the API file from config.json
            LoadConfig();

            //upload the file to be converted
            JObject uploadResults = RunUpload(input).Result;

            //Get the configuration information
            string fileID = (string)uploadResults.SelectToken("fileId");
            string fileExtension = (string)uploadResults.SelectToken("fileExtension");
            string affinityToken = (string)uploadResults.SelectToken("affinityToken");

            if(verbose)
                Console.WriteLine("Starting conversion");

            //Console.WriteLine(tester.ToString());
            JObject convertResults = RunConvert(affinityToken, fileID, "pdf").Result;

            if (verbose)
                Console.WriteLine("Conversion return");
            if (verbose)
                Console.WriteLine(convertResults.ToString());

            string processId = (string)convertResults.SelectToken("processId");
            affinityToken = (string)convertResults.SelectToken("affinityToken");

            if (verbose)
                Console.WriteLine("Affinity Token: " + affinityToken);
            if (verbose)
                Console.WriteLine("URL:  https://api.accusoft.com/v2/contentConverters/" +processId);

            //getting the results
            JObject convertStatusresults = RunConvertStatus(processId, affinityToken).Result;

            if (verbose)
                Console.WriteLine("Getting status");
            string convertStatus = (string)convertResults.SelectToken("state");
            if (verbose)
                Console.WriteLine("Status: " + convertStatus);


            while (!(convertStatus.Equals("complete")))
            {
                System.Threading.Thread.Sleep(30000);
                convertStatusresults = RunConvertStatus(processId, affinityToken).Result;
                convertStatus = (string)convertStatusresults.SelectToken("state");
                if (verbose)
                    Console.WriteLine("Conversion process: " + convertStatus);
                if (verbose)
                    Console.WriteLine((string)convertResults.SelectToken("percentComplete"));
            }

            //conversion complete - get the FileId
            string newFileID = (string)convertStatusresults.SelectToken("output.results[0].fileId");


            //download the file
            if (verbose)
                Console.WriteLine("Outputting " + output);
            RunDownloadWorkfile(affinityToken, newFileID, output).Wait();

            //for now just return true - can do errors later
            return true;
        }


        
        static async Task<JObject> RunUpload(string input)
        {
            PrizmDocConvert newUpload = new PrizmDocConvert(ApiKey);
            JObject results = await newUpload.Upload(input);
            return results;
        }


        
        static async Task<JObject> RunConvert(string affinityToken, string fileID, string format)
        {
            PrizmDocConvert newConvert = new PrizmDocConvert(ApiKey);

            JObject convertedResults =  await newConvert.Convert(affinityToken, fileID, format);
            return convertedResults;
        }

        static async Task<JObject> RunConvertStatus(string processId, string affinityToken)
        {
            PrizmDocConvert newConvert = new PrizmDocConvert(ApiKey);

            //GET /v2/contentConverters/{processId}
            JObject convertedResults = await newConvert.ConvertStatus(processId, affinityToken);
            return convertedResults;
        }

        static async Task RunDownloadWorkfile(string affinityToken, string workFile, string outfile)
        {
            PrizmDocConvert newConvert = new PrizmDocConvert(ApiKey);

            Console.WriteLine("Starting HTMLConvert Check");

            //GET /v2/contentConverters/{processId}
            await newConvert.DownloadWorkfile(affinityToken, workFile, outfile);
            
        }

        /*
         * Steps:
         * 
         * 1.  Take in two file argument - the file to be converted, and the file name to convert to.
         * 2.  
         * 
         * */

        static void Main(string[] args)
        {

            Console.WriteLine("Starting the conversion program.");

            if(args.Length != 2)
            {
                Console.WriteLine("Please enter two arguments - the name of the file to be read " +
                                    "and the name of the file to write to.  Format is PDF by default.");
            }
            

            //this is the file to be converted
            string fileToConvert = args[0];

            //this is the file to be saved
            string fileToSave = args[1];

            ConvertFileToPDF(fileToConvert, fileToSave, true);
            
            
            Console.WriteLine("Program complete.  Please press any key to exit.");
            Console.ReadKey();
        }

    }
}
